{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core/bulkactions/bulk_action_trigger

    Renders a trigger element for a given bulk action.

    Context variables required for this template:
    * showindropdown - Whether the action is displayed under a 'More' dropdown or as a separate button.
    * isfirst - Whether the action is the first one in the list.

    Example context (json):
    {
        "showindropdown": false,
        "isfirst": true
    }
}}
{{#showindropdown}}
    <a class="dropdown-item" href="#" data-action="{{$action}}{{/action}}" role="menuitem">{{$title}}{{/title}}</a>
{{/showindropdown}}
{{^showindropdown}}
    <button
        type="button"
        class="btn py-0 d-flex flex-column"
        data-action="{{$action}}{{/action}}"
        title="{{$title}}{{/title}}"
        {{^isfirst}}tabindex="-1"{{/isfirst}}
    >
        <span class="w-100 ps-2">{{$icon}}{{/icon}}</span>
        <span>{{$title}}{{/title}}</span>
    </button>
{{/showindropdown}}
