{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core/bulkactions/bulk_actions

    Renders a section with all available bulk actions.

    Context variables required for this template:
    * actions - Array of objects with the following properties:
        * actiontrigger - The HTML for the action trigger.
    * hasmoreactions - Whether there are more actions than the ones displayed.
    * moreactions - Array of objects with the following properties:
        * actiontrigger - The HTML for the action trigger.

    Example context (json):
    {
        "actions": [
            {
                "actiontrigger": "<button>Move</button>"
            }
        ],
        "hasmoreactions": true,
        "moreactions": [
            {
                "actiontrigger": "<button>Delete</button>"
            }
        ]
    }
}}
<div class="justify-content-between px-3 py-2" data-type="bulkactions">
    <div class="d-flex">
    </div>
    <div data-for="bulktools">
        <div class="btn-toolbar" role="toolbar" aria-label="{{#str}} bulkactions, core {{/str}}" data-for="bulkactions">
            {{#actions}}
                {{{actiontrigger}}}
            {{/actions}}
            {{#hasmoreactions}}
                <div class="dropup">
                    <button
                        class="btn py-0 d-flex flex-column"
                        type="button"
                        id="more-actions-toggle-{{uniqid}}"
                        data-bs-toggle="dropdown"
                        aria-haspopup="true"
                        aria-expanded="false"
                        tabindex="-1"
                    >
                        <span class="w-100 ps-2">
                            {{#pix}} i/menu, core {{/pix}}
                        </span>
                        <span>{{#str}} moremenu, core {{/str}}</span>
                    </button>
                    <div class="dropdown-menu" aria-labelledby="more-actions-toggle-{{uniqid}}" role="menu">
                        {{#moreactions}}
                            {{{actiontrigger}}}
                        {{/moreactions}}
                    </div>
                </div>
            {{/hasmoreactions}}
        </div>
    </div>
    <div class="d-flex flex-column">
        <div class="ms-auto">
            <button
                type="button"
                class="btn pe-0 pb-0"
                data-action="bulkcancel"
                data-for="bulkcancel"
                title="{{#str}} bulkcancel, core {{/str}}"
            >
                {{#pix}} e/cancel, core {{/pix}}
            </button>
        </div>
        <div data-for="bulkcount" role="status">
            &nbsp;{{#str}} bulkselection, core, {{bulkselectioncount}} {{/str}}
        </div>
    </div>
</div>
